import { LoadingDefault, BaseMessage, I18nBaseService } from 'totvs-log-base-foundation';
import { ComponentFactoryResolver, ApplicationRef, Injector, EmbeddedViewRef, Injectable } from '@angular/core';
import { LoadingComponent } from './loading.component';

@Injectable()
export class LoadingService extends LoadingDefault {

    /**
     * Referência para o componente de loading no html
     */
    private loadingComponentRef = undefined;

    /**
     * Texto apresentado durante o loading
     */
    private loadingText = '';

    /**
     * Construtor do serviço
     * @param i18n Serviço de internacionalização
     * @param componentFactoryResolver Serviço que permite adicionar componentes dinamicamente
     * @param appRef Referência da aplicação Angular
     * @param injector Serviço de injeção de dependência
     */
    constructor(
        private i18n: I18nBaseService,
        private componentFactoryResolver: ComponentFactoryResolver,
        private appRef: ApplicationRef,
        private injector: Injector) {
        super();
    }

    /**
     * Exibe o loading, bloqueando a tela
     */
    public showLoading() {
        if (this.loadingComponentRef === undefined) {
            this.createLoadingComponent();
        }

        this.loadingComponentRef.instance.showLoading();
    }

    /**
     * Esconde o loading, desbloqueando a tela
     */
    public hideLoading() {
        if (this.loadingComponentRef !== undefined) {
            this.loadingComponentRef.instance.hideLoading();
        }
    }

    /**
     * Atribui o texto que será apresentado
     * @param text Texto a ser apresentado, devendo ser um BaseMessage
     */
    public setLoadingText(text: BaseMessage) {
        this.loadingText = this.i18n.text(text) ? this.i18n.text(text).text : '';

        if (this.loadingComponentRef === undefined) {
            this.createLoadingComponent();
        }

        this.loadingComponentRef.instance.text = this.loadingText;
    }

    /**
     * Método privado que cria dinamicamente o componente de loading no DOM
     */
    private createLoadingComponent() {

        if (this.loadingComponentRef === undefined) {

            this.loadingComponentRef = this.componentFactoryResolver
                .resolveComponentFactory(LoadingComponent)
                .create(this.injector);

            this.appRef.attachView(this.loadingComponentRef.hostView);

            const domElem = (this.loadingComponentRef.hostView as EmbeddedViewRef<any>)
                .rootNodes[0] as HTMLElement;

            document.body.appendChild(domElem);
        }
    }
}
